from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import LogoutView
from django.urls import path, include, re_path
from django_spaghetti.views import Plate
from rest_framework import routers
from schema_graph.views import Schema

from sharix_admin.apiviews import *
from sharix_admin.views import *

# START Backend
# TODO: move to backend repo
router = routers.SimpleRouter()
router.register(r'sharix-users', SharixUserMVS)
router.register(r'group', GroupMVS)
# END Backend

app_name = "sharix_admin"

urlpatterns = [
    # Главная
    path('', login_required(MainView.as_view()), name='main'),
    path('support-chat', login_required(SupportChatView.as_view()), name='support_chat'),


    # --- Ниже страницы, требующие ревью ---

    path('transactions/', login_required(TransactionsView.as_view()), name='transactions'),
    path('payment/', login_required(PaymentView.as_view()), name='payment'),

    # Страница "Сотрудничество" (запрос на подключение к сервису)
    path('cooperate-request/', login_required(CooperateView.as_view()), name='cooperate_request'),
    path('job-request/', login_required(JobRequestMetaserviceView.as_view()), name='job_request_metaservice'),
    #path('job-request/<str:partner>', login_required(JobRequestPartnerView.as_view()), name='job_request_partner'),

    # Набор страниц по модели Партнер
    path('partners/', login_required(PartnersListView.as_view()), name='partners'),
    path('partner/', login_required(PartnerDetailView.as_view()), name='partner_detail_my'),
    path('partner/<str:pk>', login_required(PartnerDetailView.as_view()), name='partner_detail'),
    path('partner/edit/<int:pk>', login_required(PartnerEditView.as_view()), name='partner/edit'),
    path('partner/doc/<str:doc_code>/upload', login_required(PartnerDocUploadView.as_view()),
         name='partner_doc_upload'),
    path('partner/doc/<str:doc_code>', login_required(PartnerDocView.as_view()), name='partner_doc'),
    path('partner/change_status/', change_partner_status, name='partner/change_status'),
    
    #Набор страниц по модели Ресурс
    path('resources/', login_required(ResourcesListView.as_view()), name='resources'),
    #path('resource/<str:pk>', login_required(ResourceDetailView.as_view()), name='resource_detail'),
    path('resource/add/', login_required(ResourceCreate.as_view()), name='resource/add/'),
#    path('resource/edit/<int:pk>', login_required(ResourceEditView.as_view()), name='resource/edit'),
#    path('resource/doc/<str:doc_code>/upload', login_required(ResourceDocUploadView.as_view()),
#         name='partner_doc_upload'),
#    path('resource/doc/<str:doc_code>', login_required(ResourceDocView.as_view()), name='resource_doc'),
    path('resource/change_status/', change_resource_status, name='resource/change_status'),

    #path('resource_list/', login_required(ResourceListView.as_view()), name='resource_list'),
    #path('resource_edit/', login_required(ResourceEditView.as_view()), name='resource_edit'),

    # Набор страниц по модели Исполнитель
    path('resource_list/', login_required(ResourceListView.as_view()), name='resource_list'),
    path('resource_edit/', login_required(ResourceEditView.as_view()), name='resource_edit'),

    path('providers/', login_required(ProvidersListView.as_view()), name='providers'),
    # path('provider/<str:pk>', login_required(ProviderDetailView.as_view()), name='provider_detail'),
    # path('provider/edit/<int:pk>', login_required(ProviderEditView.as_view()), name='provider/edit'),
    # path('provider/doc/<str:doc_code>/upload', login_required(ProviderDocUploadView.as_view()),
    #      name='partner_doc_upload'),
    # path('provider/doc/<str:doc_code>', login_required(ProviderDocView.as_view()), name='provider_doc'),
    path('provider/change_status/', change_provider_status, name='provider/change_status'),

    path('service_tariff/', login_required(ServiceTariffListView.as_view()), name='service_tariff'),
    path('service_tariffs/', login_required(ServiceTariffsListView.as_view()), name='service_tariffs'),
    # path('service_tariff/<int:pk>', login_required(ServiceTariffDetailView.as_view()), name='service_tariff_detail'),

    path('service_tariff/add/', login_required(ServiceTariffCreate.as_view()), name='service_tariff/add/'),
    path('service_tariff/edit/<int:pk>', login_required(ServiceTariffUpdateView.as_view()),
         name='service_tariff/edit/'),
    path('service_tariff/change_status/', change_service_status, name='service_tariff/change_status'),

    path('service_categories/', login_required(ServiceCategoriesListView.as_view()), name='service_categories'),
    #path('service_category/<int:pk>', login_required(ServiceCategoryDetailView.as_view()), name='service_category'),
    path('service_category/edit/<int:pk>', login_required(ServiceCategoryUpdateView.as_view()), name='service_category/edit/'),
    path('service_category/add/', login_required(ServiceCategoryCreate.as_view()), name='service_category/add/'),
    path('service_category/delete/<int:pk>', login_required(ServiceCategoryDelete.as_view()), name='service_category/delete/'),
    path('service_category/change_status/', change_service_status, name='service_category/change_status'),

#CHECK Maybe obsolete
    #path('service_information/add/', login_required(ServiceInformationCreate.as_view()),
    #     name='service_information-add'),
    #path('service_information/edit/<int:pk>', login_required(ServiceInformationUpdateView.as_view()),
    #     name='service_information/edit/'),

    #path('service/', ServiceListView.as_view(), name='service'),

#TODO - take from platform
    path('access-control', login_required(AccessControlListView.as_view()), name='access_control'),
#    path('access_control/change_status/', change_access_status, name='access_control/change_status'),
#    path('relationship', login_required(RelationshipListView.as_view()), name='relationship'),    
#    path('relationship/change_status/', change_access_status, name='relationship/change_status'),
#    path('active_users', login_required(ActiveUsersListView.as_view()), name='active_users'),
#    path('active_users/change_status/', change_access_status, name='active_users/change_status'),
#    path('changelog', login_required(ChangelogListView.as_view()), name='changelog'),
#    path('closing-documents', login_required(ClosingDocumentsListView.as_view()), name='closing_documents'),


#TODO - check what is better
    # path('partner_information/', login_required(PartnerInfoView.as_view()), name='partner_information/'),
    # path('partner_information/add/', login_required(PartnerInformationCreate.as_view()), name='partner_information/add/'),
    # path('partner_information/edit/<int:pk>', login_required(PartnerInformationUpdateView.as_view()), name='partner_information/edit/'),

    #path('user_information', login_required(UserListView.as_view()), name='user_information'),

    # TODO: Посмотреть, может тоже нужно переместить
    path('senderphone/', PhoneSender.as_view()),

    # schemas
    path('schemav1/', login_required(Schema.as_view()), name='schemav1'),
    path('schemav2/', login_required(Plate.as_view()), name='schemav2'),
    path('schemav3/', schema_v3, name='schema'),

    re_path(r'^redoc/$', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),

    # START Backend
    # TODO: move to backend repo

    # Регистрация / Вход / Выход
    path('auth/signup/', ShariXSignUpView.as_view(), name="auth_signup"),
    path('auth/login/', ShariXLoginView.as_view(), name='auth_login'),
    path('auth/logout/', login_required(LogoutView.as_view()), name="auth_logout"),
    path('auth/reset-password/', ShariXResetPasswordView.as_view(), name='auth_reset_password'),

    path('transactions/<int:trans_id>/', trans_id, name='transid'),
    path('balance/', balance, name='balance'),

    path('api/v1/auth/', include('djoser.urls.authtoken'), name="api-auth"),
    path('api/v1/platform/', include(router.urls), name="api-platform"),
    path('by_phone/<phone_number>/', get_user_by_phone_number, name='get_user_by_phone_number'),

    # END Backend
]
