from datetime import datetime, timedelta
from dbsynce.lib.core import parse_requirements, get_admin_url
from dbsynce.models import Documents
from django.contrib import messages
from django.db import transaction
from django.urls import reverse_lazy
from django.views.generic.edit import FormView
from tickets.models import Ticket, TicketList

from sharix_admin.forms import CompanyForm
from sharix_admin.utils import *
from dbsynce.lib.data import *
from .base import BaseView

from dbsynce.lib.ticket_gen import *
from datetime import datetime, timedelta
from sharix_admin.forms import JobRequestPartnerForm
from sharix_admin.forms import JobRequestMetaserviceForm


class CooperateView(BaseView, FormView):
    form_class = CompanyForm
    template_name = "sharix_admin/requests/cooperate.html"
    success_url = reverse_lazy("sharix_admin:main")
    page_title = 'Сотрудничество'
    page_name = 'cooperate'

    # Проверяем не состояит ли текущий пользователь в группе PARTNER-ADMIN
    def test_func(self):
        return not "PARTNER-ADMIN" in self.user_groups

    def form_valid(self, form):
        with transaction.atomic():
            # Сохраняем форму, чтобы получить объект компании
            instance = form.save(commit=False)

            # Присваиваем полю repr идентификатор текущего пользователя
            instance.repr = self.request.user

            # Создаем новую запись в БД, чтобы иметь доступ к ID
            instance.save()

            # Создание тикета на активацию партнера.
            # Создаем объект тикета и присваиваем его полю ticket_status
            instance.ticket_status = create_ticket_company_activation(self.request.user, instance)

            # Создание тикета на утверждение прав пользователя

            Permissions.objects.create(
                id_permissions=permissions_type["partner_admin"],
                check_level=permissions_type["metaservice_admin"],
                status=activity_status["deactivated"],
                ticket_status=create_ticket_role_activation_partner_admin(self.request.user, instance)
            )

            Relationship.objects.create(
                user_who=get_user_model().objects.filter(groups__name='METASERVICE-ADMIN').first(),
                user_whom=self.request.user,
                neg_type=neg_type["contract"],
                #проверить корректность requirements
                requirements="DS01y04y05y06y07yMPUCS13MPUB",
                status=activity_status["deactivated"],
                ticket_status=create_ticket_contract_with_metaservice(self.request.user, instance)
            )

            # Сохраняем новые изменения
            instance.save()

            # Создаем необходимые объекты документов по requirements указанных в созданной company
            # Используем bulk_create для создания всех объектов одновременно
            doc_codes = parse_requirements(instance.requirements)
            Documents.objects.bulk_create([
                Documents(
                    company=instance,
                    user=self.request.user,
                    doc_type=doc_code
                ) for doc_code in doc_codes
            ])

        # Отправляем пользователю уведомление на страницу о успехе операции
        messages.success(
            self.request,
            'Ваша заявка на становление партнером успешно отправлена и теперь проходит проверку!'
        )

        return super().form_valid(form)  # Возвращаем успешный ответ


class JobRequestMetaserviceView(BaseView, FormView):
    form_class = JobRequestMetaserviceForm
    template_name = "sharix_admin/requests/job_request.html"
    success_url = reverse_lazy("sharix_admin:main")
    page_title = 'Заявка на работу в Сервисе'
    page_name = 'main'

    #def test_func(self):
    #FIXME! Check is not valid
    #    return not Practice.objects.filter(created_by=self.request.user).exists()

    def form_valid(self, form):
        with transaction.atomic():
            selected_roles = form.cleaned_data['multiple_choice_field_select']

            if 'METASERVICE-SUPERVISOR' in selected_roles:
                create_ticket_role_activation_metaservice_supervisor(self.request.user)
            if 'METASERVICE-SUPPORT' in selected_roles:
                create_ticket_role_activation_metaservice_support(self.request.user)
            if 'METASERVICE-TECHSUPPORT' in selected_roles:
                create_ticket_role_activation_metaservice_techsupport(self.request.user)

        #send_notify_mail(
        #    f'Новая заявка на прохождение практики ({instance.created_by})',
        #    "sharix_admin/emails/team_request.html",
        #    ["info@sharix-app.org"],
        #    {"ticket": ticket}
        #)

        messages.success(self.request, 'Ваша заявка успешно отправлена и теперь проходит проверку!')
        return super().form_valid(form)

class JobRequestPartnerView(BaseView, FormView):
    form_class = JobRequestPartnerForm
    template_name = "sharix_admin/requests/job_request.html"
    success_url = reverse_lazy("sharix_admin:main")
    #TODO FIXME! This page should be about particular partner!
    page_title = 'Заявка на сотрудничество с Партнером Сервиса'
    page_name = 'main'
    
    #def test_func(self):
    #FIXME! Check is not valid
    #    return not Practice.objects.filter(created_by=self.request.user).exists()
    
    def form_valid(self, form):
        with transaction.atomic():
            selected_roles = form.cleaned_data['multiple_choice_field_select']
            
            if 'PARTNER-SUPERVISOR' in selected_roles:
                create_ticket_role_activation_partner_supervisor(self.request.user)
            if 'PARTNER-SUPPORT' in selected_roles:
                create_ticket_role_activation_partner_support(self.request.user)
            if 'PARTNER-TECHSUPPORT' in selected_roles:
                create_ticket_role_activation_partner_techsupport(self.request.user)    
            if 'PROVIDER' in selected_roles:
                create_ticket_role_activation_provider(self.request.user)
                
        #send_notify_mail(
        #    f'Новая заявка на прохождение практики ({instance.created_by})',
        #    "sharix_admin/emails/team_request.html",
        #    ["info@sharix-app.org"],
        #    {"ticket": ticket}
        #)
        
        messages.success(self.request, 'Ваша заявка успешно отправлена и теперь проходит проверку!')
        return super().form_valid(form)
